@@ -131,6 +131,7 @@ gconf_key_key        (const gchar* key)
  *  Random stuff 
  */
 
+#if HAVE_CORBA
 GConfValue* 
 gconf_value_from_corba_value(const ConfigValue* value)
 {
@@ -611,6 +612,7 @@ gconf_schema_from_corba_schema(const ConfigSchema* cs)
   
   return sc;
 }
+#endif /* HAVE_CORBA */
 
 const gchar* 
 gconf_value_type_to_string(GConfValueType type)
@@ -2140,6 +2142,7 @@ gconf_value_encode (GConfValue* val)
   return retval;
 }
 
+#ifdef HAVE_CORBA
 
 /*
  * Locks
@@ -2526,7 +2529,7 @@ gconf_get_server (gboolean  start_if_not_found,
   return server;
 }
 
-GConfLock*
+static GConfLock *
 gconf_get_lock_or_current_holder (const gchar  *lock_directory,
                                   ConfigServer *current_server,
                                   GError      **err)
@@ -2793,13 +2796,18 @@ gconf_get_daemon_dir (void)
       const char *tmpdir;
 
       subdir = g_strconcat ("gconfd-", g_get_user_name (), NULL);
-      
-      if (g_getenv ("GCONF_TMPDIR")) {
-	tmpdir = g_getenv ("GCONF_TMPDIR");
-      } else {
-	tmpdir = g_get_tmp_dir ();
-      }
-      
+
+      if (g_getenv ("GCONF_TMPDIR"))
+        tmpdir = g_getenv ("GCONF_TMPDIR");
+      else if (g_getenv ("XDG_RUNTIME_DIR"))
+        {
+          g_free (subdir);
+          subdir = g_strdup ("gconfd");
+          tmpdir = g_getenv ("XDG_RUNTIME_DIR");
+        }
+      else
+        tmpdir = g_get_tmp_dir ();
+
       s = g_build_filename (tmpdir, subdir, NULL);
 
       g_free (subdir);
@@ -2911,6 +2919,8 @@ gconf_CORBA_Object_hash (gconstpointer key)
   return retval;
 }
 
+#endif /* HAVE_CORBA */
+
 void
 _gconf_init_i18n (void)
 {
