@@ -65,6 +65,8 @@
 
 #include <dbus/dbus-glib-lowlevel.h>
 
+static void logfile_remove (void);
+
 #ifdef G_OS_WIN32
 #include <io.h>
 #include <conio.h>
@@ -149,6 +151,9 @@ static void    enter_shutdown          (void);
 
 static void                 init_databases (void);
 static void                 shutdown_databases (void);
+#ifdef HAVE_DBUS
+static void                 reload_databases (void);
+#endif
 static void                 set_default_database (GConfDatabase* db);
 static void                 register_database (GConfDatabase* db);
 static void                 unregister_database (GConfDatabase* db);
@@ -171,6 +176,12 @@ static gboolean in_shutdown = FALSE;
  */
 static gboolean need_db_reload = FALSE;
 
+/*
+ * Flag indicating whether to prepare for respawn or logout
+ * when exiting
+ */
+static gboolean clean_shutdown_requested = FALSE;
+
 #ifdef HAVE_CORBA
 /* 
  * CORBA goo
@@ -343,6 +354,7 @@ gconfd_shutdown(PortableServer_Servant servant, CORBA_Environment *ev)
   
   gconf_log(GCL_DEBUG, _("Shutdown request received"));
 
+  clean_shutdown_requested = TRUE;
   gconfd_main_quit();
 }
 #endif /* HAVE_CORBA */
@@ -353,8 +365,8 @@ gconfd_shutdown(PortableServer_Servant servant, CORBA_Environment *ev)
 
 /* This needs to be called before we register with OAF
  */
-static void
-gconf_server_load_sources(void)
+static GConfSources *
+gconf_server_get_default_sources(void)
 {
   GSList* addresses;
   GList* tmp;
@@ -406,8 +418,7 @@ gconf_server_load_sources(void)
       /* don't request error since there aren't any addresses */
       sources = gconf_sources_new_from_addresses(NULL, NULL);
 
-      /* Install the sources as the default database */
-      set_default_database (gconf_database_new(sources));
+      return sources;
     }
   else
     {
@@ -446,12 +457,21 @@ gconf_server_load_sources(void)
       if (!have_writable)
         gconf_log(GCL_WARNING, _("No writable configuration sources successfully resolved. May be unable to save some configuration changes"));
 
-        
-      /* Install the sources as the default database */
-      set_default_database (gconf_database_new(sources));
+      return sources;
     }
 }
 
+static void
+gconf_server_load_sources(void)
+{
+  GConfSources* sources;
+
+  sources = gconf_server_get_default_sources();
+
+  /* Install the sources as the default database */
+  set_default_database (gconf_database_new(sources));
+}
+
 /*
  * Signal handlers should not log debug messages as this code is non-reentrant.
  * Please avoid calling gconf_log in this function.
@@ -477,6 +497,8 @@ signal_handler (int signo)
      */
     enter_shutdown ();
 
+    clean_shutdown_requested = FALSE;
+
     /* let the fatal signals interrupt us */
     --in_fatal;
     
@@ -488,6 +510,8 @@ signal_handler (int signo)
   case SIGTERM:
     enter_shutdown ();
 
+    clean_shutdown_requested = TRUE;
+
     /* let the fatal signals interrupt us */
     --in_fatal;
     
@@ -514,6 +538,7 @@ signal_handler (int signo)
 #endif
     
   default:
+    clean_shutdown_requested = FALSE;
 #ifndef HAVE_SIGACTION
     signal (signo, signal_handler);
 #endif
@@ -562,6 +587,10 @@ bus_message_handler (DBusConnection *connection,
                               DBUS_INTERFACE_LOCAL,
                               "Disconnected"))
     {
+      /* Since the log file is per-session, we should make sure it's
+       * removed when the session is over.
+       */
+      clean_shutdown_requested = TRUE;
       gconfd_main_quit ();
       return DBUS_HANDLER_RESULT_HANDLED;
     }
@@ -980,9 +1009,13 @@ main(int argc, char** argv)
 
 #ifdef HAVE_CORBA
   /* Save current state in logfile (may compress the logfile a good
-   * bit)
+   * bit) if we're exiting but may respawn.  Clean up the logfile, if
+   * the session is going away.
    */
-  logfile_save ();
+  if (clean_shutdown_requested)
+    logfile_remove ();
+  else
+    logfile_save ();
 #endif
   
   shutdown_databases ();
@@ -1030,12 +1063,13 @@ periodic_cleanup_timeout(gpointer data)
       need_db_reload = FALSE;
 #ifdef HAVE_CORBA
       logfile_save ();
-#endif
       shutdown_databases ();
       init_databases ();
       gconf_server_load_sources ();
-#ifdef HAVE_CORBA
       logfile_read ();
+#endif
+#ifdef HAVE_DBUS
+      reload_databases ();
 #endif
     }
   
@@ -1268,6 +1302,10 @@ drop_old_databases(void)
       
       if (db->listeners &&                             /* not already hibernating */
           gconf_listeners_count(db->listeners) == 0 && /* Can hibernate */
+#ifdef HAVE_DBUS
+          db->listening_clients &&
+          g_hash_table_size (db->listening_clients) == 0 &&
+#endif
           (now - db->last_access) > (60*20))           /* 20 minutes without access */
         {
           dead = g_list_prepend (dead, db);
@@ -1319,6 +1357,56 @@ shutdown_databases (void)
   default_db = NULL;
 }
 
+#ifdef HAVE_DBUS
+static void
+reload_databases (void)
+{
+  GConfSources* sources;
+  GList *tmp_list;
+
+  sources = gconf_server_get_default_sources ();
+  gconf_database_set_sources (default_db, sources);
+
+  tmp_list = db_list;
+  while (tmp_list)
+    {
+      GConfDatabase* db = tmp_list->data;
+      GList *l;
+      GConfSource *source;
+      GSList *addresses = NULL;
+      GError *error = NULL;
+
+      if (db == default_db)
+	{
+	  tmp_list = g_list_next (tmp_list);
+	  continue;
+	}
+
+      for (l = db->sources->sources; l != NULL; l = l->next)
+        {
+          source = l->data;
+          addresses = g_slist_prepend (addresses, source->address);
+        }
+
+      addresses = g_slist_reverse (addresses);
+      sources = gconf_sources_new_from_addresses (addresses, &error);
+
+      if (error == NULL)
+        {
+          gconf_database_set_sources (db, sources);
+        }
+      else
+        {
+          /* if we got an error, keep our old sources -- that's better than
+           * nothing */
+          g_error_free (error);
+        }
+
+      tmp_list = g_list_next (tmp_list);
+    }
+}
+#endif
+
 static gboolean
 no_databases_in_use (void)
 {
@@ -1571,6 +1659,30 @@ gconfd_check_in_shutdown (CORBA_Environment *ev)
  * Logging
  */
 
+static const char *
+get_session_guid (void)
+{
+  const char *session_bus_address;
+  const char *guid;
+
+  /* FIXME: we may want to use dbus-address.h functions here
+   */
+  session_bus_address = g_getenv ("DBUS_SESSION_BUS_ADDRESS");
+
+  if (session_bus_address == NULL)
+    return NULL;
+
+  guid = g_strrstr (session_bus_address, "guid=");
+
+  if (guid == NULL)
+    return NULL;
+
+  if (guid[0] == '\0')
+    return NULL;
+
+  return guid + strlen ("guid=");
+}
+
 /*
    The log file records the current listeners we have registered,
    so we can restore them if we exit and restart.
@@ -1599,14 +1711,23 @@ gconfd_check_in_shutdown (CORBA_Environment *ev)
 static void
 get_log_names (gchar **logdir, gchar **logfile)
 {
-#ifndef G_OS_WIN32
-      const char *home = g_get_home_dir ();
-#else
-      const char *home = _gconf_win32_get_home_dir ();
-#endif
+  const char *session_guid;
+  char *state_file;
 
-  *logdir = g_build_filename (home, ".gconfd", NULL);
-  *logfile = g_build_filename (*logdir, "saved_state", NULL);
+  *logdir = gconf_get_daemon_dir ();
+
+  /* We make the state file per-session so multiple gconfds
+   * don't stomp over each other.
+   */
+  session_guid = get_session_guid ();
+
+  if (session_guid != NULL)
+    state_file = g_strdup_printf ("saved_state_%s", session_guid);
+  else
+    state_file = g_strdup ("saved_state");
+
+  *logfile = g_build_filename (*logdir, state_file, NULL);
+  g_free (state_file);
 }
 
 static void close_append_handle (void);
@@ -1829,6 +1950,21 @@ logfile_save (void)
     close (fd);
 }
 
+static void
+logfile_remove (void)
+{
+  gchar *logdir = NULL;
+  gchar *logfile = NULL;
+
+  get_log_names (&logdir, &logfile);
+
+  g_unlink (logfile);
+
+  g_free (logdir);
+  g_free (logfile);
+}
+
+
 typedef struct _ListenerLogEntry ListenerLogEntry;
 
 struct _ListenerLogEntry
